@@ -20,7 +20,7 @@
 """
 Registry of built-in language definitions.
 
-This file is imported by the :mod:`parce.registry` module, so that the global
+This file is imported by the :mod:`~parce.registry` module, so that the global
 registry contains all the language definitions listed below.
 
 This file is manually updated.
@@ -62,6 +62,14 @@ register("parce.lang.css.Css.root",
     guesses = [(r'\b@media\b', 0.5), (r'\bdiv\b', 0.1), (r'\bbody\s*\{', 0.4)],
 )
 
+register("parce.lang.csv.Csv.root",
+    name = "CSV",
+    desc = "Comma-separated values",
+    filenames = [("*.csv", 1)],
+    mimetypes = [("text/csv", 1)],
+    guesses = [(r'([^\n,"]+|[ \t]*"(""|[^"])*"[ \t]*)(,([^\n,"]+|[ \t]*"(""|[^"])*"[ \t]*))+(\n|$)', 0.1)],
+)
+
 register("parce.lang.docbook.DocBook.root",
     name = "DocBook",
     desc = "DocBook XMl/SGML computer documentation schema",
@@ -109,7 +117,12 @@ register("parce.lang.html.Html.root",
     desc = "HTML (4 or 5)",
     filenames = [("*.html", 1), ("*.htm", 1)],
     mimetypes = [("text/html", 1)],
-    guesses = [(r'(?i)<!DOCTYPE html', .9), (r'\bXHTML.*?\bTransitional/', .9),(r'(?i)<html\b', .9)],
+    guesses = [
+        (r'(?i)<!DOCTYPE html', .9),
+        (r'\bXHTML.*?\bTransitional/', .9),
+        (r'(?i)<html\b', .9),
+        (r'<[^\W\d]\w*(\s+[^\W\d]\w*(\s*=\s*(".*?"|\S+))?)*\s*/?>', .1),
+    ],
 )
 
 register("parce.lang.tex.Latex.root",
@@ -118,7 +131,7 @@ register("parce.lang.tex.Latex.root",
     aliases = ["TeX"],
     filenames = [("*.tex", 1), ("*.sty", .8), ("*.cls", .1)],
     mimetypes = [("text/x-latex", .8), ("application/x-latex", .8)],
-    guesses = [(r'\\document(class|style)\{', .8)],
+    guesses = [(r'\\document(class|style)[\{\[]', .8)],
 )
 
 register("parce.lang.lilypond.LilyPond.root",
@@ -126,7 +139,10 @@ register("parce.lang.lilypond.LilyPond.root",
     desc = "LilyPond music typesetter",
     filenames = [("*.ly", 1), ("*.ily", .8), ("*.lyi", .5)],
     mimetypes = [("text/x-lilypond", .8)],
-    guesses = [(r'\\version\s*"\d', .8)],
+    guesses = [
+        (r'\\version\s*"\d', .8),
+        (r'\\relative ', .1)
+    ],
 )
 
 register("parce.lang.numbers.Deutsch.root",
@@ -210,7 +226,12 @@ register("parce.lang.html.XHtml.root",
     desc = "HTML that is valid XML",
     filenames = [("*.html", 1), ("*.htm", 1), ("*.xhtml", 1)],
     mimetypes = [("text/html", 1), ("application/xhtml+xml", 1)],
-    guesses = [(r'(?i)<!DOCTYPE html', .8), (r'\bXHTML.*?\bStrict/', .9),(r'(?i)<html\b', .8)],
+    guesses = [
+        (r'(?i)<!DOCTYPE html', .8),
+        (r'\bXHTML.*?\bStrict/', .9),
+        (r'(?i)<html\b', .8),
+        (r'<[^\W\d]\w*(\s+[^\W\d]\w*(\s*=\s*(".*?"|\S+))?)*\s*/?>', .05),
+    ],
 )
 
 register("parce.lang.xml.Xml.root",
