@@ -27,10 +27,6 @@ context, and some toplevel convenience functions.
 
 See also the documentation: :doc:`transforming`.
 
-TODO: it would be nice to be able to specify the Transform for another Language
-inside a Transform, just like a lexicon can target a lexicon from a different
-language. But I'm not yet sure how it would be specified.
-
 """
 
 import collections
@@ -205,11 +201,14 @@ class Transformer(parce.util.Observable):
 
     """
 
-    #: This format string creates the name to look for when searching a suitable
-    #: Transform class in a Language module space (see :meth:`find_transform`).
-    #:
-    #: .. versionadded:: 0.27.0
     transform_name_template = "{}Transform"
+    """This format string creates the name to look for when searching a
+    suitable Transform class in a Language module space (see
+    :meth:`find_transform`).
+
+    .. versionadded:: 0.27.0
+
+    """
 
     def __init__(self):
         super().__init__()
@@ -227,49 +226,97 @@ class Transformer(parce.util.Observable):
         if not root_lexicon:
             return  # a root lexicon can be None, but then there are no children
 
-        from parce.tree import make_tokens  # local lookup is faster
+        from parce.tree import Context, make_tokens  # local lookup is faster
+        from parce.lexer import Event
+        from parce.target import Target
+
+        def make_target(pop, push):
+            """Return a Target if pop < 0 or push is not empty."""
+            if pop or push:
+                return Target(pop, push)
+
+        def build_tree(lexicons, events):
+            """Build a tree in case missing lexicons need to be handled.
+
+            Returns the tree and the next event (if any), with adapted pop
+            value.
+
+            """
+            tree = context = Context(lexicons[0], None)
+            for lexicon in lexicons[1:]:
+                context = Context(lexicon, context)
+                context.parent.append(context)
+            for target, lexemes in events:
+                if target:
+                    for pop in range(target.pop, 0):
+                        if context is tree:
+                            return tree, Event(make_target(pop, target.push), lexemes)
+                        context = context.parent
+                    for lexicon in target.push:
+                        context = Context(lexicon, context)
+                        context.parent.append(context)
+                context.extend(make_tokens(lexemes, context))
+            return tree, None
+
+        def consume_events(lexicons, events):
+            """Simply consume events until the first of the specified lexicons ends.
+
+            Returns the next event, if any, with adapted pop value.
+
+            """
+            pop = len(lexicons)
+            for target, lexemes in events:
+                if target:
+                    pop += target.pop
+                    if pop <= 0:
+                        return Event(make_target(pop, target.push), lexemes)
+                    pop += len(target.push)
 
         curlang = root_lexicon.language
         transform = self.get_transform(curlang)
 
-        items = Items(root_lexicon.arg)
-        stack = []
         events = parce.lexer.Lexer([root_lexicon]).events(text, pos)
-        lexicon = root_lexicon
+        root_meth = getattr(transform, root_lexicon.name, None)
+        if root_meth:
+            add_untransformed = _allow_untransformed(root_meth)
+            items = Items(root_lexicon.arg)
+            stack = []
+            lexicon = root_lexicon
 
-        def get_object_item(items):
-            """Get the object item, may update curlang and transform variables."""
-            nonlocal curlang, transform
-            if curlang is not lexicon.language:
-                curlang = lexicon.language
-                transform = self.get_transform(curlang)
-            name = lexicon.name
-            meth = getattr(transform, name, None)
-            if meth:
-                return Item(name, meth(items))
+            for target, lexemes in events:
+                while target:
+                    for _ in range(target.pop, 0):
+                        lexicon, olditems, meth, name, add_untransformed = stack.pop()
+                        olditems.append(Item(name, meth(items)))
+                        items = olditems
+                    for i, l in enumerate(target.push):
+                        if l.language is not curlang:
+                            curlang = l.language
+                            transform = self.get_transform(curlang)
+                        meth = getattr(transform, l.name, None)
+                        if meth:
+                            stack.append((lexicon, items, meth, l.name, add_untransformed))
+                            add_untransformed = _allow_untransformed(meth)
+                            items = Items(l.arg)
+                            lexicon = l
+                        else:
+                            if add_untransformed:
+                                context, event = build_tree(target.push[i:], events)
+                                items.append(Item("<untransformed>", context))
+                            else:
+                                event = consume_events(target.push[i:], events)
+                            target, lexemes = event if event else (None, ())
+                            break
+                    else:
+                        break
+                items.extend(make_tokens(lexemes))
 
-        for target, lexemes in events:
-            if target:
-                for _ in range(target.pop, 0):
-                    item = get_object_item(items)
-                    lexicon, items = stack.pop()
-                    if item:
-                        items.append(item)
-                for l in target.push:
-                    stack.append((lexicon, items))
-                    items = Items(l.arg)
-                    lexicon = l
-            items.extend(make_tokens(lexemes))
-
-        # unwind
-        while stack:
-            item = get_object_item(items)
-            lexicon, items = stack.pop()
-            if item:
-                items.append(item)
-        item = get_object_item(items)
-        if item:
-            return item.obj
+            # unwind
+            while stack:
+                lexicon, olditems, meth, name, add_untransformed = stack.pop()
+                olditems.append(Item(name, meth(items)))
+                items = olditems
+            return root_meth(items)
 
     def transform_tree(self, tree):
         """Evaluate a tree structure."""
@@ -282,6 +329,7 @@ class Transformer(parce.util.Observable):
         transform = self.get_transform(curlang)
         root_meth = getattr(transform, tree.lexicon.name, None)
         if root_meth:
+            add_untransformed = _allow_untransformed(root_meth)
             stack = []
             node, items, i = tree, Items(tree.lexicon.arg), 0
             while not self._interrupt[tree]:
@@ -301,13 +349,16 @@ class Transformer(parce.util.Observable):
                             try:
                                 items.append(Item(name, self._cache[n]))
                             except KeyError:
-                                stack.append((items, i + 1, meth))
+                                stack.append((items, i + 1, meth, add_untransformed))
                                 node, items, i = n, Items(n.lexicon.arg), 0
+                                add_untransformed = _allow_untransformed(meth)
                                 break
+                        elif add_untransformed:
+                            items.append(Item("<untransformed>", n))
                 else:
                     if stack:
                         name = node.lexicon.name
-                        olditems, i, meth = stack.pop()
+                        olditems, i, meth, add_untransformed = stack.pop()
                         obj = self._cache[node] = meth(items)
                         items = olditems
                         items.append(Item(name, obj))
@@ -404,15 +455,24 @@ class Transformer(parce.util.Observable):
         self.build(builder.root)
 
     def get_transform(self, language):
-        """Return a Transform class instance for the specified language."""
+        """Return a Transform class instance for the specified language.
+
+        May return None, if no Transform was added and none could be found.
+
+        """
         return self._transforms[language]
 
     def add_transform(self, language, transform):
-        """Add a Transform instance for the specified language."""
+        """Add a Transform instance for the specified language.
+
+        You may also specify None, to disable transformation for that language
+        even if a transform could automatically be found.
+
+        """
         self._transforms[language] = transform
 
     def find_transform(self, language):
-        """If no Transform was added, try to find a predefined one.
+        """Try to find a Transform for the specified language definition.
 
         This is done by looking for a Transform subclass in the language's
         module, with the same name as the language with "Transform" appended.
@@ -422,6 +482,9 @@ class Transformer(parce.util.Observable):
         This naming scheme can be modified by setting the
         :attr:`transform_name_template` attribute.
 
+        This method is called by :meth:`get_transform` if no Transform was
+        added for the language.
+
         """
         module = sys.modules[language.__module__]
         tfname = self.transform_name_template.format(language.__name__)
@@ -466,3 +529,32 @@ def transform_text(root_lexicon, text, transform=None, pos=0):
     return t.transform_text(root_lexicon, text, pos)
 
 
+def add_untransformed(func):
+    """Decorator to mark a Transform method with the 'add_untransformed' flag.
+
+    When a :class:`Transform` method has this flag, child contexts of this
+    method's context that have no transform method are not silently ignored,
+    but added in an :class:`Item` with ``name`` ``<untransformed>`` (including
+    the angle brackets).
+
+    For example::
+
+        class MyLangTransform(Transform):
+            @add_untransformed
+            def root(self, items):
+                "This method also gets the untransformed Contexts"
+                for item in items:
+                    if not item.is_token and item.name == "<untransformed>":
+                        context = item.obj
+                        # do something with the parce context....
+
+    """
+    func.add_untransformed = True
+    return func
+
+
+def _allow_untransformed(meth):
+    """Return True when the method wants the untransformed stuff."""
+    return getattr(meth.__func__, "add_untransformed", False)
+
+
